/*
   Loads a previous saved alarmlist created by Alarmlist_Save.rexx
   and add the programs into TView's Alarms window.

   Version 1.1  10/nov/2003
   Dagfinn Olsen (dagfolse@online.no)

   TView will be iconified during the operation, to speed up the process.
   Also, from you start the script, until TView gets iconified, dont make
   use of mouse&Keyboard!

   Requires following files:
   =========================
   RexxEvent1_21.lha                            installed and runing as commodity
   RexxTricks_386.lha                           rexxtricks.library in Libs:
   SetWin              from SetWin1.0.lha       script asumes it's in C:
   MultiRequestChoice  from MRChoice.lha        script asumes it's in C:


   Version history:
   ---------------
   v1.1 fixed support for channel names built of more than 2 words,
	now script supports a channelname built of max 6 words. I hope
	there are no channel that uses more then 6 words in their name :)

   v1.0 initial release

   Todo list:
   ---------
   -do not add programs that has ended or already started.




   ==================================================
   = Change the next few lines to match your system =
   ==================================================

*/


/* Change this to where you have saved the alarmlist, keep both "'" intact */
tmpfile = 'S:tview_alarm.list'

/* Change path to where you placed the exe's. Keep both "'" intact */
setwinprg = 'C:SetWin'
mrchoiceprg = 'C:MultiRequestChoice'

/* If you want to see some debug output, change 0 -> 1 and run script from shell */
debug = 0




/* ========================== */
/* No more modifying required */
/* ========================== */




options results
options failat 21

call addlib('rexxtricks.library',0,-30,0)


/* check if alarmlist have been readed before */
status = GETCOMMENT(tmpfile)

IF status = 0 THEN DO
   arguments = '"Info" "Saved alarmlist has been loaded before.\nLoad it again?" "Yes|No"'
   address command mrchoiceprg arguments
   svar = RC

   IF debug = 1 THEN DO
      SAY ''
      SAY 'Yes=1, No=0 >> 'svar
      END

   /* want to load old alarmlist again */
   IF svar = 1 THEN DO
      IF debug = 1 THEN DO
	 SAY ''
	 SAY '**YES, WANT TO LOAD AGAIN**'
	 SAY ''
	 END

      call lesedata

      IF debug = 1 THEN DO
	 SAY ''
	 SAY 'Script finished, all seems ok'
	 SAY ''
	 END
      exit
   END


   /* don't want to load old alarmlist, quiting */
   IF svar = 0 THEN DO
      IF debug = 1 THEN DO
	 SAY ''
	 SAY '**DONT WANT TO RELOAD OLD LIST, QUITING**'
	 SAY ''
	 END
      exit
   END

END

/* loading new alarmlist  */
ELSE DO
   IF debug = 1 THEN DO
      SAY ''
      SAY '**LOADING NEW ALARMLIST**'
      SAY ''
      END
   call lesedata

   /* mark alarmlist as "loaded" */
   address command
   'filenote 'tmpfile' 0'
   IF debug = 1 THEN DO
      SAY ''
      SAY 'Script finished, all seems ok'
      SAY ''
      END
   exit
END





/* ****************************************************** */
/* Sub program: parse textfile                            */
/* ****************************************************** */
lesedata:

     address command
     'resident c:wait'
     setwinprg||' "Alarms" ACTIVATE FRONT'
     sjekk = RC

     IF sjekk = 10 THEN DO
		   SAY '**Error from setwin is OK, when Alarmlist window is not open**'
		   setwinprg||' "TView#?" ACTIVATE FRONT'
		   'wait 1'
	    END
	
     ELSE DO
	  setwinprg||' "Alarms" CLOSE'
	  setwinprg||' "TView#?" ACTIVATE FRONT'
	  'wait 1'
	    END

     address REXXEVENT
     "<ctrl tab>"
     address command
     'wait 1'
     address REXXEVENT
     "<tab>"
     address command
     'wait 1'
     address REXXEVENT
     "<shift up>"
     address command
     'wait 1'
     address REXXEVENT
     "<ctrl tab>"
     address command
     'wait 1'

     address tview.1
     'hide'
     'getsliderval'
     slider = RESULT

     CALL READFILE(tmpfile,'array')

     IF debug = 1 THEN DO
	 SAY ''
	 SAY '---------INFO_ABOUT_LOADED_ALARMLIST-----------------'
	 SAY 'Number of lines:' array.0
	 DO i = 1 TO array.0
	    SAY 'Line' i || ':' array.i
	 END
	 SAY '---------END--OF--LIST-------------------------------'
	 SAY ''
     END

     DO i = 1 TO array.0
       IF debug = 1 THEN DO
		 SAY ''
		 SAY '============================================='
		 SAY 'Reading line: 'i
       END

       /* Channelname is 1 word  */  CALL READLINES(tmpfile,,,'%s %s %50c','ch_a1 clock_a program_a')
       /* Channelname is 2 words */  CALL READLINES(tmpfile,,,'%s %s %s %50c','ch_b1 ch_b2 clock_b program_b')
       /* Channelname is 3 words */  CALL READLINES(tmpfile,,,'%s %s %s %s %50c','ch_c1 ch_c2 ch_c3 clock_c program_c')
       /* Channelname is 4 words */  CALL READLINES(tmpfile,,,'%s %s %s %s %s %50c','ch_d1 ch_d2 ch_d3 ch_d4 clock_d program_d')
       /* Channelname is 5 words */  CALL READLINES(tmpfile,,,'%s %s %s %s %s %s %50c','ch_e1 ch_e2 ch_e3 ch_e4 ch_e5 clock_e program_e')
       /* Channelname is 6 words */  CALL READLINES(tmpfile,,,'%s %s %s %s %s %s %s %50c','ch_f1 ch_f2 ch_f3 ch_f4 ch_f5 ch_f6 clock_f program_f')

       /* *** Channel name is 1 word *** */
       IF MATCHPATTERN(clock_a.i,'#?:#?') THEN DO
		 IF debug = 1 THEN DO
		       SAY 'Checking if parsed string is time-string  "'||clock_a.i||'"'
		       SAY 'A: String was time.'
		       SAY 'Channel: 'ch_a1.i
		       SAY 'Channelname is 1 word'
		       SAY 'Time   : 'clock_a.i
		       SAY 'Program: 'program_a.i
		       SAY '--------------------------------------'
		 END

		 tid = clock_a.i
		 txt = program_a.i
		 std = ch_a1.i
		 CALL merking
       END /* *** Channel name is 1 word *** */

       ELSE DO

	  /* *** Channel name is 2 words *** */
	  IF MATCHPATTERN(clock_b.i,'#?:#?') THEN DO
		    IF debug = 1 THEN DO
		       SAY 'Checking if parsed string is time-string  "'||clock_b.i||'"'
		       SAY 'A: String was time.'
		       SAY 'Channel: 'ch_b1.i ch_b2.i
		       SAY 'Channelname is 2 words'
		       SAY 'Time   : 'clock_b.i
		       SAY 'Program: 'program_b.i
		       SAY '--------------------------------------'
		    END

		    tid = clock_b.i
		    txt = program_b.i
		    std = ch_b1.i ch_b2.i
		    CALL merking
	  END /* *** Channel name is 2 words *** */

	  ELSE DO

	     /* *** Channel name is 3 words *** */
	     IF MATCHPATTERN(clock_c.i,'#?:#?') THEN DO
		       IF debug = 1 THEN DO
			       SAY 'Checking if parsed string is time-string  "'||clock_c.i||'"'
			       SAY 'A: String was time.'
			       SAY 'Channel: 'ch_c1.i ch_c2.i ch_c3.i
			       SAY 'Channelname is 3 words'
			       SAY 'Time   : 'clock_c.i
			       SAY 'Program: 'program_c.i
			       SAY '--------------------------------------'
		       END

		       tid = clock_c.i
		       txt = program_c.i
		       std = ch_c1.i ch_c2.i ch_c3.i
		       CALL merking
	     END /* *** Channel name is 3 words *** */

	     ELSE DO

		/* *** Channel name is 4 words *** */
		IF MATCHPATTERN(clock_d.i,'#?:#?') THEN DO
			  IF debug = 1 THEN DO
			       SAY 'Checking if parsed string is time-string  "'||clock_d.i||'"'
			       SAY 'A: String was time.'
			       SAY 'Channel: 'ch_d1.i ch_d2.i ch_d3.i ch_d4.i
			       SAY 'Channelname is 4 words'
			       SAY 'Time   : 'clock_d.i
			       SAY 'Program: 'program_d.i
			       SAY '--------------------------------------'
			  END

			  tid = clock_d.i
			  txt = program_d.i
			  std = ch_d1.i ch_d2.i ch_d3.i ch_d4.i
			  CALL merking
		END /* *** Channel name is 4 words *** */

		ELSE DO

		   /* *** Channel name is 5 words *** */
		   IF MATCHPATTERN(clock_e.i,'#?:#?') THEN DO
			     IF debug = 1 THEN DO
				       SAY 'Checking if parsed string is time-string  "'||clock_e.i||'"'
				       SAY 'A: String was time.'
				       SAY 'Channel: 'ch_e1.i ch_e2.i ch_e3.i ch_e4.i ch_e5.i
				       SAY 'Channelname is 5 words'
				       SAY 'Time   : 'clock_e.i
				       SAY 'Program: 'program_e.i
				       SAY '--------------------------------------'
			     END

			     tid = clock_e.i
			     txt = program_e.i
			     std = ch_e1.i ch_e2.i ch_e3.i ch_e4.i ch_e5.i
			     CALL merking
		   END /* *** Channel name is 5 words *** */

		   ELSE DO

		      /* *** Channel name is 6 words *** */
		      IF MATCHPATTERN(clock_f.i,'#?:#?') THEN DO
			     IF debug = 1 THEN DO
				       SAY 'Checking if parsed string is time-string  "'||clock_f.i||'"'
				       SAY 'A: String was time.'
				       SAY 'Channel: 'ch_f1.i ch_f2.i ch_f3.i ch_f4.i ch_f5.i ch_f6.i
				       SAY 'Channelname is 6 words'
				       SAY 'Time   : 'clock_f.i
				       SAY 'Program: 'program_f.i
				       SAY '--------------------------------------'
			     END

			     tid = clock_f.i
			     txt = program_f.i
			     std = ch_f1.i ch_f2.i ch_f3.i ch_f4.i ch_f5.i ch_f6.i
			     CALL merking
		      END /* *** Channel name is 6 words *** */

		   END
		END
	     END
	  END
       END
     END

     address tview.1
     'show'
     'setsliderval 12'
     'setsliderval 'slider

     return
/* ****************************************************** */










/* ****************************************************** */
/* Sub program: add items to alarmlist window             */
/* ****************************************************** */
merking:
 address tview.1
 'setsliderval 'RANDOM(0,23)
 'Search "'||txt||'"'
 'GetNumberItems'
 items=RESULT
 IF debug = 1 THEN SAY 'TView searched and found this many: 'items

 IF items > 1 THEN DO
      IF debug = 1 THEN SAY 'C: Found more than 1, checking further'
      DO j=0 to items-1
	   IF debug = 1 THEN SAY 'C-'j': Now checking number: 'j
	   address tview.1
	   'SetActive 'j
	   'GetTime'
	   time=RESULT
	   'GetChannel'
	   sted=RESULT
	   IF sted = std THEN DO
	      IF debug = 1 THEN SAY 'C-'j': Same channel, checking time.'
	      IF time = tid THEN DO
		 IF debug = 1 THEN SAY 'C-'j': Same channel & Same time. Added in alarmlist'
		 address tview.1
		 'SetActive 'j
		 'Alarm'
		 LEAVE
		 END
		 ELSE DO
		    IF debug = 1 THEN SAY 'C-'j': Same channel, but not correct time. Skipping this one'
		 END
	      END
	      ELSE DO
		    IF debug = 1 THEN SAY 'C-'j': Not correct channel. Skipping this one'
	      END
      END
 END
 ELSE DO
    IF items = 1 THEN DO
       IF debug = 1 THEN SAY 'D: The searched program has been added in alarmlist.'
       'SetActive 0'
       'Alarm'
    END
    IF items = 0 THEN DO
       IF debug = 1 THEN SAY 'D: Nothing found that equals this line. No entry in alarmlist'
    END
 END

return
/* ****************************************************** */

